//
//  Scheme.swift
//  uTune
//
//  Created by Kevin Schneier on 3/11/19.
//

import UIKit
import os.log

class Scheme: NSObject, NSCoding {
    
    // MARK: Properties
    
    var name: String
    var frequencies: [Double]
    
    
    // MARK: Archiving Paths
    static let DocumentsDirectory = FileManager().urls(for: .documentDirectory, in: .userDomainMask).first!
    static let ArchiveURL = DocumentsDirectory.appendingPathComponent("schemes")
    
    
    // MARK: Types
    
    struct PropertyKey {
        static let name = "name"
        static let frequencies = "frequencies"
    }
    
    // MARK: Initialization
    
//    init?(name: String, s1: Double, s2: Double, s3: Double, s4: Double, s5: Double, s6: Double){
    init?(name: String, frequencies: [Double]){
        guard !name.isEmpty else {
            return nil
        }

        self.name = name
        self.frequencies = frequencies
    }
    
    
    // MARK: NSCoding
    func encode(with aCoder: NSCoder) {
        aCoder.encode(name, forKey: PropertyKey.name)
        aCoder.encode(frequencies, forKey: PropertyKey.frequencies)
    }
    
    required convenience init?(coder aDecoder: NSCoder) {
        // The name is required. If we cannot decode a name string, the initializer should fail.
        guard let name = aDecoder.decodeObject(forKey: PropertyKey.name) as? String else {
            os_log("Unable to decode the name for a Scheme object.", log: OSLog.default, type: .debug)
            return nil
        }
        
        guard let frequencies = aDecoder.decodeObject(forKey: PropertyKey.frequencies) as? [Double] else {
            os_log("Unable to decode the frequencies for a Scheme object.", log: OSLog.default, type: .debug)
            return nil
        }
        
        self.init(name: name, frequencies: frequencies)
    }
}
