//
//  SchemeTableViewController.swift
//  uTune
//
//  Created by Kevin Schneier on 3/11/19.
//

import UIKit
import os.log

class SchemeTableViewController: UITableViewController {
    
    //MARK: Properties
    var schemes = [Scheme]()
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        // Use the edit button item provided by the table view controller
        navigationItem.leftBarButtonItem = editButtonItem
        
        // Load any saved schemes, otherwise load sample data.
        if let savedSchemes = loadSchemes() {
            schemes += savedSchemes
        }
        else {
            // Load the sample data
            loadSampleSchemes()
        }
        
    }
    
//    override func viewDidAppear(_ animated: true) {
//
//    }
    
    // MARK: - Table view data source
    
    override func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }
    
    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return schemes.count
    }
    
    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {

        let cellIdentifier = "SchemeTableViewCell"
        
        guard let cell = tableView.dequeueReusableCell(withIdentifier: cellIdentifier, for: indexPath) as? SchemeTableViewCell else {
            fatalError("The dequeued cell is not an instance of SchemeTableViewCell.")
        }
        
        
        let scheme = schemes[indexPath.row]
        cell.schemeLabel.text = scheme.name
        
        return cell
    }
    
    
    // Override to support conditional editing of the table view.
    override func tableView(_ tableView: UITableView, canEditRowAt indexPath: IndexPath) -> Bool {
        // Return false if you do not want the specified item to be editable.
        return true
    }
    
    // Override to support editing the table view.
    override func tableView(_ tableView: UITableView, commit editingStyle: UITableViewCell.EditingStyle, forRowAt indexPath: IndexPath) {
        if editingStyle == .delete {
            // Delete the row from the data source
            schemes.remove(at: indexPath.row)
            saveSchemes()
            tableView.deleteRows(at: [indexPath], with: .fade)
        } else if editingStyle == .insert {
            // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view
        }
    }
    
    /*
     // Override to support rearranging the table view.
     override func tableView(_ tableView: UITableView, moveRowAt fromIndexPath: IndexPath, to: IndexPath) {
     
     }
     */
    
    /*
     // Override to support conditional rearranging of the table view.
     override func tableView(_ tableView: UITableView, canMoveRowAt indexPath: IndexPath) -> Bool {
     // Return false if you do not want the item to be re-orderable.
     return true
     }
     */
    
    // MARK: - Navigation
    
    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        
        super.prepare(for: segue, sender: sender)
        
        switch(segue.identifier ?? "") {
            
        case "AddItem":
            os_log("Adding a new scheme.", log: OSLog.default, type: .debug)
            
        case "ShowDetail":
            guard let schemeDetailViewController = segue.destination as? SchemeViewController else {
                fatalError("Unexpected destination: \(segue.destination)")
            }
            
            guard let selectedSchemeCell = sender as? SchemeTableViewCell else {
                fatalError("Unexpected sender: \(sender)")
            }
            
            guard let indexPath = tableView.indexPath(for: selectedSchemeCell) else {
                fatalError("The selected cell is not being displayed by the table")
            }
            
            let selectedScheme = schemes[indexPath.row]
            schemeDetailViewController.scheme = selectedScheme
            
            // THIS WAS PUT IN FOR THE CANCEL BUTTON
            schemeDetailViewController.segueUsed = "ShowDetail"
            
            
        default:
            fatalError("Unexpected Segue Identifier; \(segue.identifier)")
        }
    }
    
    
    //MARK: Actions
    @IBAction func unwindToSchemeList(sender: UIStoryboardSegue) {
        
        if let sourceViewController = sender.source as? SchemeViewController, let scheme = sourceViewController.scheme {
            
            if let selectedIndexPath = tableView.indexPathForSelectedRow {
                
                // Update an existing scheme.
                schemes[selectedIndexPath.row] = scheme
                tableView.reloadRows(at: [selectedIndexPath], with: .none)
            }
            else {
                // Add a new scheme.
                let newIndexPath = IndexPath(row: schemes.count, section: 0)
                
                schemes.append(scheme)
                tableView.insertRows(at: [newIndexPath], with: .automatic)
            }
            
            // Save the schemes.
            saveSchemes()
            
        }
    }
    
    //MARK: Private Methods
    private func loadSampleSchemes() {
        guard let scheme1 = Scheme(name: "EADGBE", frequencies: [82.41, 110.0, 146.8, 196.0, 246.9, 329.6]) else {
            fatalError("Unable to instantiate scheme1")
        }
        
        guard let scheme2 = Scheme(name: "DADGAD", frequencies: [73.42, 110.0, 146.8, 196.0, 220.0, 293.7]) else {
            fatalError("Unable to instantiate scheme2")
        }
        
        schemes += [scheme1, scheme2]
    }
    private func saveSchemes() {
        // +archivedDataWithRootObject:requiringSecureCoding:error:
        
        let isSuccessfulSave = NSKeyedArchiver.archiveRootObject(schemes, toFile: Scheme.ArchiveURL.path)
        
        
        if isSuccessfulSave {
            os_log("Schemes successfully saved.", log: OSLog.default, type: .debug)
        } else {
            os_log("Failed to save meals...", log: OSLog.default, type: .error)
        }
        
    }
    
    private func loadSchemes() -> [Scheme]? {
        return NSKeyedUnarchiver.unarchiveObject(withFile: Scheme.ArchiveURL.path) as? [Scheme]
    }
    
    
}
