//
//  SchemeViewController.swift
//  uTune
//
//  Created by Kevin Schneier on 3/11/19.
//

import UIKit
import os.log
import CoreBluetooth


class SchemeViewController: UIViewController, UITextFieldDelegate, UINavigationControllerDelegate, CBPeripheralManagerDelegate, UIPickerViewDelegate, UIPickerViewDataSource {
    
    //MARK: Properties
    @IBOutlet weak var schemeTextField: UITextField!
    @IBOutlet weak var saveButton: UIBarButtonItem!
    @IBOutlet weak var tuneButton: UIButton!
    
    // Scheme variable
    var scheme: Scheme?
    
    // Frequency vector variable to be put in scheme (start with default values for EADGBE)
    var frequencies: [Double] = [82.41, 110.0, 146.8, 196.0, 246.9, 329.6]
    
    // THIS WAS PUT IN FOR THE CANCEL BUTTON
    var segueUsed: String?
    
    // Bluetooth stuff
    var peripheralManager: CBPeripheralManager?
    var peripheral: CBPeripheral!
    var connectionStatus = false
    
    // Picker View stuff
    var schemePicker: UIPickerView!
    var pickerData = [["E", "D\u{266F}", "D", "C\u{266F}", "C", "B"], ["A", "G\u{266F}", "G", "F\u{266F}", "F"], ["D", "C\u{266F}", "C", "B", "A\u{266F}"], ["G", "F\u{266F}", "F", "E", "D\u{266F}"], ["B", "A\u{266F}", "A", "G\u{266F}"], ["E", "D\u{266F}", "D", "C\u{266F}", "C"]]
    var totalFrequencies = [[82.41, 77.78, 73.42, 69.30, 65.41, 61.74], [110.0, 103.8, 98.01, 92.51, 87.31], [146.8, 138.6, 130.8, 123.5, 116.5], [196.0, 185.0, 174.6, 164.8, 155.6], [246.9, 233.1, 220.0, 207.7], [329.6, 311.1, 293.7, 277.2, 261.6]]

    
    
    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view, typically from a nib.
        
        // Handle the text field's user input through delegate callbacks
        schemeTextField.delegate = self
        
        // Set up views if editing an existing scheme.
        if let scheme = scheme {
            navigationItem.title = scheme.name
            schemeTextField.text   = scheme.name
            frequencies = scheme.frequencies
        }

        // Update Connection Status
        if blePeripheral?.state == CBPeripheralState.connected {
            connectionStatus = true
        }
        
        // Update save and tune buttons
        updateSaveButtonState()
        updateTuneButtonState()
        
        // Create and start the peripheral manager
        peripheralManager = CBPeripheralManager(delegate: self, queue: nil)
    }
    
    override func viewDidAppear(_ animated: Bool) {
        // Update Connection Status
        if blePeripheral?.state == CBPeripheralState.connected {
            connectionStatus = true
        }
        else {
            connectionStatus = false
        }
        // Update tune button
        updateTuneButtonState()
        
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
        
        // THIS IS OPTIONAL
        // Remove this view controller from the stack
//        if segueUsed == "ShowDetail" {
//            if let nav = self.navigationController {
//                let vcs = nav.viewControllers.filter {(vc) in
//                    return (vc as? SchemeViewController) == nil
//                }
//
//                self.navigationController?.setViewControllers(vcs, animated: false)
//            }
//        }
    }
    
    
    func createPicker(_ textField : UITextField){
        
        // Create picker at bottom of screen
        self.schemePicker = UIPickerView(frame:CGRect(x: 0, y: 0, width: self.view.frame.size.width, height: 350))
        self.schemePicker.delegate = self
        self.schemePicker.dataSource = self
        self.schemePicker.backgroundColor = UIColor.white
        schemeTextField.inputView = self.schemePicker
        
        // Create ToolBar to go above picker
        let toolBar = UIToolbar()
        toolBar.barStyle = .default
        toolBar.isTranslucent = true
        toolBar.sizeToFit()
        
        // Add buttons to the toolbar
        let doneButton = UIBarButtonItem(title: "Done", style: .plain, target: self, action: #selector(SchemeViewController.doneClick))
        let spaceButton = UIBarButtonItem(barButtonSystemItem: .flexibleSpace, target: nil, action: nil)
        toolBar.setItems([spaceButton, doneButton], animated: false)
        toolBar.isUserInteractionEnabled = true
        textField.inputAccessoryView = toolBar
        
        // Preselect the correct rows
        for i in 0...frequencies.count-1 {
            let correctRow = totalFrequencies[i].index(of: frequencies[i])
            schemePicker.selectRow(correctRow!, inComponent: i, animated: false)
        }
    }
    
    @objc func doneClick() {
        var string = ""
        for index in 0...frequencies.count-1 {
            string = string + pickerData[index][schemePicker.selectedRow(inComponent: index)]
            frequencies[index] = totalFrequencies[index][schemePicker.selectedRow(inComponent: index)]
        }
        schemeTextField.text = string
        schemeTextField.resignFirstResponder()
    }
    
    func textFieldDidBeginEditing(_ textField: UITextField) {
        // Disable the Save button while editing.
        saveButton.isEnabled = false
        tuneButton.isHidden = true
        createPicker(schemeTextField)
    }
    
    func textFieldDidEndEditing(_ textField: UITextField) {
        updateSaveButtonState()
        updateTuneButtonState()
        
        navigationItem.title = textField.text
    }
    
    //MARK: Picker
    func numberOfComponents(in pickerView: UIPickerView) -> Int {
        return pickerData.count
    }
    
    func pickerView(_ pickerView: UIPickerView, numberOfRowsInComponent component: Int) -> Int {
        return pickerData[component].count
    }
    
    func pickerView(_ pickerView: UIPickerView, titleForRow row: Int, forComponent component: Int) -> String? {
        return pickerData[component][row]
    }
    
    func pickerView(_ pickerView: UIPickerView, didSelectRow row: Int, inComponent component: Int) {
        // This method is triggered whenever the user makes a change to the picker selection.
        // The parameter named row and component represents what was selected.
    }
    
    func pickerView(_ pickerView: UIPickerView, viewForRow row: Int, forComponent component: Int, reusing view: UIView?) -> UIView {
        var label = UILabel()
        if let v = view {
            label = v as! UILabel
        }
        label.font = UIFont (name: "Helvetica Neue", size: 16)
        label.text =  pickerData[component][row]
        label.textAlignment = .center
        return label
    }
    
    
    //MARK: Navigation
    @IBAction func cancel(_ sender: UIBarButtonItem) {
        // Depending on style of presentation (modal or push presentation), this view controller needs to be dismissed in two different ways.
        
        // I CHANGED SOME STUFF HERE TO MAKE THE CANCEL BUTTON WORK
        if segueUsed == "ShowDetail" {
            navigationController?.popViewController(animated: true)
        }
        else {
            dismiss(animated: true, completion: nil)
        }
    }
    
    
    override func prepare(for segue: UIStoryboardSegue, sender: Any?){
        
        super.prepare(for:segue, sender:sender)
        
        // Configure the destination view controller only when the save button is pressed.
        guard let button = sender as? UIBarButtonItem, button === saveButton else {
            os_log("The save button was not pressed, cancelling", log: OSLog.default, type: .debug)
            return
        }
        
        let name = schemeTextField.text ?? ""
        
        scheme = Scheme(name: name, frequencies: frequencies)
        
    }
    
    
    //MARK: Actions
    @IBAction func pressTuneButton(_ sender: Any) {
        for index in 0...frequencies.count-1 {
            writeValue(data: frequencies[index].description + " ")
        }
//        writeValue(data: "\r\n")
    }
    
    
    
    //MARK: Private Methods
    private func updateSaveButtonState() {
        // Disable the Save button if the text field is empty.
        let text = schemeTextField.text ?? ""
        saveButton.isEnabled = !text.isEmpty
    }
    
    private func updateTuneButtonState() {
        // Disable the tune button if the text field is empty
        let text = schemeTextField.text ?? ""
        tuneButton.isHidden = text.isEmpty || !connectionStatus
    }
    
    
    // Write functions
    func writeValue(data: String){
        let valueString = (data as NSString).data(using: String.Encoding.utf8.rawValue)
        //change the "data" to valueString
        if let blePeripheral = blePeripheral{
            if let txCharacteristic = txCharacteristic {
                blePeripheral.writeValue(valueString!, for: txCharacteristic, type: CBCharacteristicWriteType.withResponse)
            }
        }
    }
    
    func peripheralManagerDidUpdateState(_ peripheral: CBPeripheralManager) {
        if peripheral.state == .poweredOn {
            return
        }
        print("Peripheral Manager is running")
    }
    
}

